/***************************************************************************
                          Attribute.cpp  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//**** Attribute class

#include "Attribute.h"

/** The Attribute constructor should be initialized with the name
    of the attribute and the magnitude. */
Attribute::Attribute(int iAttribute, int iMagnitude) : attribute(iAttribute)
{
   // Get the text version of the attribute name
   attributeName = parseAttribute(iAttribute);
   // Set the magnitude
   setMagnitude(iMagnitude);
   // Set the table values for our extended attribute data
   setExtData();
}
/** Copy constructor */
Attribute::Attribute(const Attribute& rhs) : Modifiable(rhs)
{
   if (&rhs != this) {
      attribute = rhs.attribute;
      attributeName = rhs.attributeName;
      for (int i=0; i < 4; i++)
         extData[i] = rhs.extData[i];
   }
}

Attribute::~Attribute()
{
}

/** Set the unmodified magnitude to newMagnitude, making sure that it is not greater than the maximum allowed */
int Attribute::setMagnitude(const int newMagnitude)
{
   if ( newMagnitude > ATTRIBUTE_MAXIMUM && attribute != MOVEMENT) { magnitude = ATTRIBUTE_MAXIMUM; }
   else { magnitude = newMagnitude; }
   /** Make sure the magnitude is not 0, which is an illegal value */
   if (magnitude <= 0) magnitude=1;
   /** Set the extended data appropriatly */
   setExtData();
   return magnitude;
}
/** Set the extended attribute data from the attribute type and the score */
void Attribute::setExtData(){
   // Get a data table
   dLoader* table = new dLoader();
   // Get the table values for our attribute and score
   int *ext = table->getextData(attribute, magnitude);

   // Set the extended data.
   for (int i=0; i < 4; i++)
      extData[i] = 0;

   // Set the extended data.
   if ( attribute == MOVEMENT ) {
      extData[0] = ext[magnitude-1];
   }
   else
      for (int i=0; i < table->getextData(attribute); i++)
         extData[i] = ext[i];
	
   // delete our data loader
   delete table;
}
/** Get the extended attribute data */
int Attribute::getExtData(int data){
   return (extData[data]);
}
/** Sum all of the modifiers and assign that value to the attribute permanently */
int Attribute::setMagnitude()
{
   magnitude += sumOfAllModifiers();
   if (magnitude > ATTRIBUTE_MAXIMUM && attribute != MOVEMENT) { magnitude = ATTRIBUTE_MAXIMUM; }
   if ( magnitude <= 0) magnitude=1;
   modifiers.clear();
   /** Set the extended data appropriatly */
   setExtData();
   return (magnitude);
}
/** Get and set the string name of the attribute from the int identifier */
string Attribute::parseAttribute(const int iName)
{
	dLoader* table = new dLoader();
	string *anlt = table->getAttributeNameLookupTable();
	delete (table);
	attributeName = anlt[iName];
	return (anlt[iName]);
}


#ifdef _DEBUG_
void Attribute::print_out() {
         cout << "Name = " << attributeName << " : ident = " << attribute;
         for (int i=0; i<4; i++)
            cout << " : extdata[" << i << "] = " << extData[i];
         cout << endl;
         Modifiable *ptr = this;
         ptr->print_out();
}
#endif


