/***************************************************************************
                          Modifier.cpp  -  description
                             -------------------
    begin                : Tue Nov 13 2001
    copyright            : (C) 2001 by Sheldon Lee Wen
    email                : tormak@home.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Modifier.h"
#include <stdio.h>
#include <iostream.h>

/**
 * Modifier class. This class represents a modifier. It allows you to add and
 * remove scopes in which the modifier is active, and set that scope on or off.
 * In the future it will need an event handling interface so that scopes can be
 * automatically set on and off.
 * @author Sheldon Lee Wen
 */

Modifier::Modifier(int Imagnitude, int Iscope, bool IInScope) : magnitude(Imagnitude)
{
	addScope(Iscope, IInScope);
}

Modifier::~Modifier()
{
    if (source != NULL) source->removeModifier(this);
    data.clear();
}

bool Modifier::setScope(int ExistingScope, bool newScopeValue)
{  // If the scope exists, update the scope.
   if ( (data.find(ExistingScope)) != data.end() ) {
      data[ExistingScope] = newScopeValue;
      updateScope();
      return (data[ExistingScope]);
   }
   return (false);
}

bool Modifier::getScope(const int ExistingScope)
{
   if ( (data.find(ExistingScope)) != data.end() )
      return data[ExistingScope];
   else return(false);
}

/** inScope() updates the InScope variable and returns it. */
bool Modifier::inScope()
{
  	updateScope();
  	return (InScope);
}

/** The  InScope variable is set to true if any scope in the data map
	  is in scope. It is set to false otherwise. */
void Modifier::updateScope()
{
   InScope = false;
   hash_map<int,bool>::iterator it = data.begin();
   for (; it != data.end(); it++)
      if (it->second == true)
         InScope=true;
}

// Stuff to handle the ScopePair list
void Modifier::clear()
{
   data.clear();
}

bool Modifier::containsKey(const int key)
{
   if ( data.find(key) != data.end() )
	return (true);
	
   return(false);
}
void Modifier::removeScope(const int IScope)
{
   data.erase(IScope);
}

bool Modifier::addScope(int ind, bool val)
{
   if ( data.find(ind) == data.end() ) {
      data[ind]=val;
      return (true); }
   else
      return (false);
}

/** operator +  */
int Modifier::operator + (Modifier &rhs){
	if (inScope() && rhs.inScope())
		return (magnitude +  rhs.getValue());
	return magnitude;
}

/** operator +  */
int Modifier::operator + (const int &rhs){
	if (inScope())
		return (magnitude +  rhs);
	return magnitude;
}

void Modifier::handleModifierEvent(modifierEvent &evt) {
#ifdef _DEBUG_
  __DEBUG(Handling the Event);
  ___DEBUG(Setting scope to, evt.getValue());
#endif
      setScope(evt.getScope(),evt.getValue());
}
