/***************************************************************************
                          Modifier.h  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//**** Header for Modifier Class

#ifndef MODIFIER_H
#define MODIFIER_H

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <algorithm>
#include <hash_map>
#include "modifiereventlistener.h"
#include "modifiable.h"

class Modifiable;

/**   @author Sheldon Lee-Wen
      The modifier is a bonus or penalty to a character attribute that is applicable in certain
      circumstances, or "scopes." A modifier can have more than one scope, but it makes no
      sense to have more than one of the same scope. A modifier can be applied to any class
      that inherits the modifiable class, which provides methods for adding, removing, and updating
      modifiers as well as summing the total values of modifiers.
      */
class Modifier : public modifierListener{

public:

  Modifier(int Imagnitude, int Iscope, bool IInScope);
  ~Modifier();

  /** Set the magnitude of the modifier */
  int setValue(const int newValue) { return(magnitude = newValue); }
  /** Returns the value of magnitude */
  int getValue() const { return magnitude; }
  /** Sets a scope ExistingScope to be in or out of scope */
  bool setScope(int ExistingScope, bool newScopeValue);
  /** Returns whether the scope ExistingScope is in scope or not.
  		If the scope ExistingScope does not exist in the modifier return false */
  bool getScope(const int ExistingScope);
  /** adds a scope to the modifier. Return false if the scope already exists. */
  bool addScope(int IScope, bool IInScope);
  /*  remove a scope */
  void removeScope(const int IScope);
  /**  call updateScope() and return InScope to see if modifier is in any scope. */
  bool inScope();
  /** Set the source of the modifier */
  void setSource(Modifiable* ns) { source = ns; }
  /** clear all scopes. */
  void clear();
  bool containsKey(const int key);
  int size() { return data.size();}  // how many elements in table?

  /** operator +  */
  int operator+(Modifier &rhs );
  /** operator +  */
  int operator+(const int &rhs );

private:
  /** Handle an event */
  virtual void handleModifierEvent(modifierEvent &evt);
  /** The magnitude of the modifier */
  int magnitude;
  /** InScope is true if any of the modifiers scopes are active. It is false otherwise */
  bool InScope;
  /** data contains the scope pairs. */
  hash_map<int, bool>  data;
  /** A pointer to the object that contains the modifier. This allows us to remove modifiers from classes that contain it
       when we call the destructor. */
  Modifiable *source;

  void updateScope();

};
#endif
