/***************************************************************************
                          bodyarea.h  -  description
                             -------------------
    begin                : Tue Dec 4 2001
    copyright            : (C) 2001 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BODYAREA_H
#define BODYAREA_H

#include "modifiable.h"
#include "dataloader.h"
#include "staticdata.h"
#include <string>
#include <iostream>

/**This class represents a single body area. It supports modifiers
to the health point total of the body area and is
mutable by the Race class. It also tracks the status of
a body area.
  *@author Sheldon Lee Wen
  */

class BodyArea : public Modifiable  {
public:
   /** Default constructor */
   BodyArea();
   /** Constructor, takes the body area, and the HP maximum for that body area */
   BodyArea(int bArea, int hpTotal);
   /** Copy Constructor */
   BodyArea(const BodyArea &rhs);
   /** Operator = */
//   const BodyArea &operator=(const BodyArea &rhs);
   ~BodyArea();
   /** returns the name of the attribute in string form. */
   string getName() const;
   /** returns the integer identifier */
   inline int getIdentifier() const { return bodyarea; }
   /** sums all of the existing modifiers and assigns the value to magnitude */
   virtual int setMagnitude();
   /** Sets the magnitude to newMagnitude, limited by the maximum value that a
      body area can have, determined by race. */
   virtual int setMagnitude(const int newMagnitude);
   /** Write property of int status. */
   void setstatus( const int _newVal);
   /** Read property of int status. */
   int getstatus() const;
   /** Write property of int bodyareaMaximum. */
   void setbodyareaMaximum( const int _newVal);
   /** Read property of int bodyareaMaximum. */
   int getbodyareaMaximum() const;
   #ifdef _DEBUG_
   void print_out();
   #endif



private: // Private methods
   /** Internal method that parses out the string name of the body area from it's integer value. */
   string parseBodyArea(const int iBodyArea);
   string bodyareaName;
   int bodyarea;
   /** Maximum HP value for the body area. */
   int bodyareaMaximum;
   /** Holds the status of the body area. These aren't yet defined but will be things like,
      damaged, removed, etc. */
   int status;

   friend class Race;
   friend class Character;
};

#endif
