/***************************************************************************
                          modifiable.cpp  -  description
                             -------------------
    begin                : Thu Nov 8 2001
    copyright            : (C) 2001 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "modifiable.h"

Modifiable::Modifiable(){
}
Modifiable::Modifiable(const Modifiable& rhs){
   if (&rhs != this) {
      magnitude = rhs.magnitude;
      modifiers = rhs.modifiers;
   }
}

/** Operator overloading */
const Modifiable &Modifiable::operator=( const Modifiable & rhs) {
   if (this != &rhs ) {
      magnitude = rhs.magnitude;
      modifiers = rhs.modifiers;
   }
   return *this;
}


Modifiable::~Modifiable(){
   for (unsigned int i=0; i < modifiers.size(); i++)  {
      modifiers.erase(&modifiers[i]);
   }
   modifiers.clear();
}

/** This method adds a modifier */
void Modifiable::addModifier(Modifier *source){
   source->setSource(this);
   modifiers.push_back(source);
}
/** This method removes a modifier */
void Modifiable::removeModifier(Modifier *source){
   if (!modifiers.empty()) {
      source->setSource(NULL);
      for (unsigned int i=0; i < modifiers.size(); i++)
         if (modifiers[i] == source) modifiers.erase(&modifiers[i]);

//      modifiers.removeElement(source);
   }
}
/** This method returns the number of modifiers our object has. */
int Modifiable::numModifiers() {
   return modifiers.size();
}
/** This method totals the magnitudes of all modifiers that are in scope and returns it. */
int Modifiable::sumOfAllModifiers() {
   int num=0;
   if (!modifiers.empty())
      for (unsigned int i = 0; i < modifiers.size() ; i++)
         	if (modifiers[i] != NULL && modifiers[i]->inScope())
                	num += modifiers[i]->getValue();
   return (num);
}
/** Returns the base magnitude of the modifiable object */
int Modifiable::getMagnitude() const{
	return magnitude;
}
/** Returns the magnitude with all of it's in scope modifiers added. */
int Modifiable::getModifiedMagnitude() {
   return (magnitude + sumOfAllModifiers());
}
/** This method returns the magnitude plus the values of any in scope modifiers, plus a one time constant value modified */
int Modifiable::getModifiedMagnitude(int modified){
   return (magnitude + modified + sumOfAllModifiers());
}
/** Returns the magnitude with all of it's in scope modifiers added, plus 
a modifier passed in the variable modified. This one time modifier will 
not be added to the modifiable object. */
int Modifiable::getModifiedMagnitude(Modifier& modified) {
   return (modified + magnitude + sumOfAllModifiers());
}
/** Sums all of the existing modifiers and assigns the value to magnitude */
int Modifiable::setMagnitude(){
   magnitude += sumOfAllModifiers();
   return (magnitude);
}
/** Set the magnitude to newMagnitude. This should be overridden in
inherited classes to check that newMagnitude is valid. */
int Modifiable::setMagnitude( const int newMagnitude){
	return(magnitude = newMagnitude);
}

#ifdef _DEBUG_
void Modifiable::print_out() {
   cout << " : magnitude = " << magnitude;
   for (unsigned int i=0; i<modifiers.size(); i++)
      cout << " : modifiers[" << i << "] = " << (modifiers[i])->getValue() << " : inScope = " << (modifiers[i])->inScope();
   cout << endl;
}
#endif
