/***************************************************************************
                          race.cpp  -  description
                             -------------------
    begin                : Wed Oct 31 2001
    copyright            : (C) 2001 by Sheldon Lee Wen
    email                : tormak@home.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "race.h"
#include <cstdlib>
#include <iostream>

/** Constructor. Creates an empty race w/ undefined values. */
Race::Race()
{
	init();
}
/** Constructor */
Race::Race(const int iRace, const int iGender)
{
	init();
	setRace(iRace, iGender);
}
/** Copy Constructor */
Race::Race(const Race &iRace)
{
     init();
     const_cast<int&>(base_defense_rating) =	iRace.base_defense_rating;
	defense_rating=iRace.defense_rating;
	race = iRace.race;
	gender = iRace.gender;
	raceName = iRace.raceName;
	genderName = iRace.genderName;
		
	for (int i=0; i <NUM_ATTRIBUTES; i++)
		attributeBonus[i] = iRace.attributeBonus[i];
	for (int i=0; i <BODY_NUM_BODY; i++)
           hp[i] = iRace.hp[i];
}

/** Operator overloading */
const Race &Race::operator=( const Race & rhs) {
   if (this != &rhs ) {
      const_cast<int&>(base_defense_rating) =	rhs.base_defense_rating;
      defense_rating=rhs.defense_rating;
      race = rhs.race;
      gender = rhs.gender;
      raceName = rhs.raceName;
      genderName = rhs.genderName;
	
      for (int i=0; i <NUM_ATTRIBUTES; i++)
         attributeBonus[i] = rhs.attributeBonus[i];
      for (int i=0; i <BODY_NUM_BODY; i++)
         hp[i] = rhs.hp[i];
   }
   return *this;
}

Race::~Race() {
}

void Race::init()
{
	const_cast<int&>(base_defense_rating)=0;
	defense_rating=0;
	race=-1;
	gender=-1;
	raceName="";
	genderName="";
	for (int i=0; i <NUM_ATTRIBUTES; i++) {
		attributeBonus[i] = 0;
	}
}
	
void Race::setRace (const int nRace, const int nGender)
{
     dLoader *table = new dLoader();
	init();
	
	 //Make sure we have a valid race and gender
	assert( nRace >=0 && nRace <= NUM_RACES);
	assert( nGender >=0 && nGender <= NUM_GENDERS);

	race=nRace;
     gender=nGender;

     string *gnlt = table->getGenderNameLookupTable();
     string *rnlt = table->getRaceNameLookupTable();
	int *hpt = table->getHealthPointTable(race);
	int *drt = table->getDefenseRatingTable();
	int *attr = table->getRacialAttributeTable(race, gender);
	
     genderName=gnlt[gender];
     raceName=rnlt[race];
	hp[BODY_HEAD] = BodyArea(HEAD, hpt[HEAD]);
	hp[BODY_TORSO] = BodyArea(TORSO, hpt[TORSO]);
	hp[BODY_LEFT_ARM] = BodyArea(ARM, hpt[ARM]);
	hp[BODY_RIGHT_ARM] = BodyArea(ARM, hpt[ARM]);
	hp[BODY_LEFT_LEG] = BodyArea(LEG, hpt[LEG]);
	hp[BODY_RIGHT_LEG] = BodyArea(LEG, hpt[LEG]);
	hp[BODY_LEFT_WING] = BodyArea(WING, hpt[WING]);
	hp[BODY_RIGHT_WING] = BodyArea(WING, hpt[WING]);
	hp[BODY_TAIL] = BodyArea(TAIL, hpt[TAIL]);	

	for (int i=0; i <NUM_ATTRIBUTES; i++)
		attributeBonus[i] = attr[i];
	
	const_cast<int&>(base_defense_rating) = defense_rating = drt[race];
}

#ifdef _DEBUG_
void Race::print_out() {
         cout << "raceName = " << raceName << " : ident = " << race;
         cout << " : genderName = " << genderName << " : ident = " << gender;
         cout << " : dr = " << defense_rating << " : bdr = " << base_defense_rating;
         for (int i=0; i<NUM_ATTRIBUTES; i++)
            cout << " : attributeBonus[" << i << "] = " << attributeBonus[i] << endl;
         for (int i=0; i<BODY_NUM_BODY; i++) {
            cout << " : hp[" << i << "] = " << endl;
            hp[i].print_out();                  }
}
#endif